import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonIOException;
import com.google.gson.JsonSyntaxException;

/*
 * Implementa la relació entre
 * el model DAO basat en CRUD
 * i el model de la base de dades real
 */

public class DaoAlumne implements Dao<ObjAlumne> {

    private void writeList (ArrayList<ObjAlumne> llista) {
        try {
            Gson gson = new GsonBuilder().setPrettyPrinting().create();
            String jsonInString = gson.toJson(llista);
            PrintWriter out = new PrintWriter(MainDao.alumnesPath);
            out.write(jsonInString);
            out.flush();
            out.close();
        } catch (JsonSyntaxException | JsonIOException | FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    private int getPosition (int id) {
        int result = -1;
        ArrayList<ObjAlumne> llista = getAll();
        for (int cnt = 0; cnt < llista.size(); cnt = cnt + 1) {
            ObjAlumne alumne = llista.get(cnt);
            if (alumne.getId() == id) {
                result = cnt;
                break;
            }
        }
        return result;
    }

    @Override
    public void add(ObjAlumne alumne) {
        ArrayList<ObjAlumne> llista = getAll();
        ObjAlumne item = get(alumne.getId());
        if (item == null) {
            llista.add(alumne);
            writeList(llista);
        }
    }

    @Override
    public ObjAlumne get(int id) {
        ObjAlumne result = null;
        ArrayList<ObjAlumne> llista = getAll();
        int pos = getPosition(id);
        if (pos != -1) {
            result = llista.get(pos);
        }
        return result;
    }

    @Override
    public ArrayList<ObjAlumne> getAll() {
        ArrayList<ObjAlumne> result = null;
        try {
            Gson gson = new Gson();
            ObjAlumne[] arr = gson.fromJson(new FileReader(MainDao.alumnesPath), ObjAlumne[].class);
            result = new ArrayList<ObjAlumne>(Arrays.asList(arr));
        } catch (JsonSyntaxException | JsonIOException | FileNotFoundException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public void update(int id, ObjAlumne alumne) {
        ArrayList<ObjAlumne> llista = getAll();
        int pos = getPosition(id);
        if (pos != -1) {
            llista.set(pos, alumne);
            writeList(llista);
        }
    }

    @Override
    public void delete(int id) {
        ArrayList<ObjAlumne> llista = getAll();
        int pos = getPosition(id);
        if (pos != -1) {
            llista.remove(pos);
            writeList(llista);
        }
    }

    @Override
    public void print () {
        ArrayList<ObjAlumne> llista = getAll();
        for (int cnt = 0; cnt < llista.size(); cnt = cnt + 1) {
            System.out.println("    " + llista.get(cnt));
        }
    }
}
